/* Pong - A version of Pong based on Ncurses
 * Copyright (C) 1999 Jeffrey Wade Starr <starr81@ix.netcom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */   

// pong.cpp
// Pong - everyone's favorite game!
// Reimplemented by Jeffrey W. Starr Oct 11 1998
// History: Version 0.3 started Oct 20 1998
// History: Version 0.4 started Nov 3  1998
// History: Version 0.41 started Mar 14 1999

#include <iostream>
#include <ncurses.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include "paddle.h"
#include "random.h"
#include "ball.h"
#include "event.h"
#include "controls.h"

int kbhit();
int pausegame();

int kbhit(void)
{
  // For explanations of the code, see the select() man page

  fd_set rfds;
  struct timeval tv;
  int retval;

  // Watch stdin (fd 0) to when it has input
  FD_ZERO(&rfds);
  FD_SET(0, &rfds);

  // Wait up to 1 second (CHANGE)
  tv.tv_sec = 0;
  tv.tv_usec = 10000;

  if (select(1, &rfds, NULL, NULL, &tv))
    return true;
  else
    return false;
  // Don't rely on the value of tv now!
}

int pausegame()
{
  char c;
  while(true)
    {
      if( kbhit() )
	{
	  c = getch();
	  if(c == 'p')
	    return 0;
	}
    }
}

int main(int argc, char *argv[])
{
  initscr();
  crmode();
  noecho();
  curs_set(1);
  
  Keybmod(argc, argv);

  Random * rnd = new Random( (unsigned) time (NULL));
  int dirct = rnd->next(3) + 1;

  Paddle * paddle = new Paddle;
  Ball * ball = new Ball(40, 5, dirct);
  Event * event = new Event;

  char c;

  do
    {
      if( kbhit() )
	{
	  c = getch();
	  if(c == LEFT)
	    paddle->mvLeft();
	  if(c == RIGHT)
	    paddle->mvRight();
	  if(c == PAUSE)
	    pausegame();
	  if(c == FASTER)
	    faster();
	  if(c == SLOWER)
	    slower();
	  if(c == REALSLOW)
	    timetick = 500000;  // really slow down the game
	  if(c == QUIT)
	    break;                // hopefully this will break the while
	}
    }
  while( ok(rnd, ball, paddle, event) );

  nocrmode();
  echo();
  endwin();

  cout << "End of Game" << endl;
  cout << "Your score: " << event->getScore() << endl << endl;
}




